'use strict';

var path = require('path');
var fs = require('fs');

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

/**
 * @description 是否是文件夹
 * @param path
 * @returns
 */
var isDir = function (path) {
    return fs.lstatSync(path).isDirectory();
};
/**
 * @description 是否存在文件
 * @param path
 * @returns
 */
var hasFile = function (path) {
    return fs.existsSync(path);
};
/**
 * @description 获取所有文件夹
 * @param path
 * @returns
 */
var getDirs = function (path$1) {
    var dirs = fs.readdirSync(path$1);
    return dirs.reduce(function (result, name) {
        var fullPath = path.join(path$1, name);
        isDir(fullPath) && result.push({ dirName: name, dirPath: fullPath });
        return result;
    }, []);
};
/**
 * @description 合并配置
 * @param source object
 * @param target object
 * @returns object
 */
function mergeConfig(source, target) {
    var isObject = function (data) { return Object.prototype.toString.call(data) === '[object Object]'; };
    var hasKey = function (data, key) { return isObject(data) && Object.prototype.hasOwnProperty.call(data, key); };
    for (var key in target) {
        if (!hasKey(source, key)) {
            source[key] = target[key];
            isObject(target[key]) && mergeConfig(source[key], target[key]);
        }
        else {
            isObject(target[key]) && mergeConfig(source[key], target[key]);
        }
    }
    return source;
}

var isFunction = function (d) { return Object.prototype.toString.call(d) === '[object Function]'; };
var ReflectPolyfill = /** @class */ (function () {
    function ReflectPolyfill() {
    }
    /**
     * @description 对象是否存在某个属性
     * @param target source
     * @param propertyKey key
     * @returns boolean
     */
    Object.defineProperty(ReflectPolyfill, "has", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (target, propertyKey) {
            return isFunction(Reflect.has) ? Reflect.has(target, propertyKey) : Object.prototype.hasOwnProperty.call(target, propertyKey);
        }
    });
    /**
     * @description 删除某个属性
     * @param target source
     * @param propertyKey key
     * @returns boolean
     */
    Object.defineProperty(ReflectPolyfill, "deleteProperty", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (target, propertyKey) {
            return isFunction(Reflect.deleteProperty) ? Reflect.deleteProperty(target, propertyKey) : delete target[propertyKey];
        }
    });
    return ReflectPolyfill;
}());
var StringPolyfill = /** @class */ (function () {
    function StringPolyfill() {
    }
    /**
     * @description 是否以某个字符串开头
     * @param str string
     * @param search string
     * @returns boolean
     */
    Object.defineProperty(StringPolyfill, "startWith", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (str, search) {
            return isFunction(String.prototype.startsWith) ? str.startsWith(search) : str.slice(0, search.length) === search;
        }
    });
    return StringPolyfill;
}());

/**
 * @description 获取json
 * @param jsonPath jsonPath
 * @returns
 */
function getJson(jsonPath) {
    try {
        var jsonText = fs.readFileSync(jsonPath, 'utf-8');
        return JSON.parse(jsonText);
    }
    catch (error) {
        process.exit(0);
    }
}
/**
 * @description 合并json
 * @param target IJson
 * @param source IJson
 * @returns IJson
 */
function mergeJson(target, source) {
    var _a, _b, _c, _d;
    var targetPaths = (_b = (_a = target.compilerOptions) === null || _a === void 0 ? void 0 : _a.paths) !== null && _b !== void 0 ? _b : {};
    var sourcePaths = (_d = (_c = source.compilerOptions) === null || _c === void 0 ? void 0 : _c.paths) !== null && _d !== void 0 ? _d : {};
    for (var pathKey in targetPaths) {
        if (!ReflectPolyfill.has(sourcePaths, pathKey)) {
            sourcePaths[pathKey] = targetPaths[pathKey];
        }
    }
    return __assign(__assign({}, source), { compilerOptions: __assign(__assign({}, source.compilerOptions), { paths: sourcePaths }) });
}
/**
 * @description 生成json
 * @param alias { [key: string]: string }
 * @param root string
 * @param prefix string
 * @returns IJson
 */
function genJson(alias, root, prefix) {
    var rootName = path.parse(root).name;
    var paths = Object.keys(alias).reduce(function (result, pathKey) {
        if (StringPolyfill.startWith(pathKey, prefix) && pathKey !== prefix) {
            var name_1 = path.parse(alias[pathKey]).name;
            result["".concat(pathKey, "/*")] = ["".concat(rootName, "/").concat(name_1, "/*")];
        }
        result["".concat(prefix, "/*")] = ["".concat(rootName, "/*")];
        return result;
    }, {});
    return {
        compilerOptions: {
            paths: paths,
            baseUrl: './'
        }
    };
}
/**
 * @description 同步json文件
 * @param extendJson 继承的json
 * @param jsJson jsconfig.json
 * @param tsJson tsconfig.json
 */
function syncJson(_a) {
    var extendJson = _a.extendJson, jsJson = _a.jsJson, tsJson = _a.tsJson, alias = _a.alias, prefix = _a.prefix, root = _a.root, mode = _a.mode;
    if (hasFile(extendJson) && ['all', 'extends'].includes(mode)) {
        var json = genJson(alias, root, prefix);
        hasFile(extendJson) && fs.writeFileSync(extendJson, JSON.stringify(json, null, 4));
    }
    if (hasFile(jsJson) && ['all', 'sync'].includes(mode)) {
        var target = genJson(alias, root, prefix);
        var source = getJson(jsJson);
        var newJson = mergeJson(target, source);
        hasFile(jsJson) && fs.writeFileSync(jsJson, JSON.stringify(newJson, null, 4));
    }
    if (hasFile(tsJson) && ['all', 'sync'].includes(mode)) {
        var target = genJson(alias, root, prefix);
        var source = getJson(tsJson);
        var newJson = mergeJson(target, source);
        hasFile(tsJson) && fs.writeFileSync(tsJson, JSON.stringify(newJson, null, 4));
    }
}

/**
 * @description 插件名称
 */
var PLUGIN_NAME = 'vue-cli-plugin-inject-alias';
/**
 * @description 处理插件默认配置
 */
var defaultConfig = function (cwd) {
    return {
        root: path.resolve(cwd, 'src'),
        prefix: '@',
        mode: 'all'
    };
};
/**
 * @description 插件配置文件路径
 */
var getPaths = (function (cwd) {
    return {
        ALIAS_JSON_PATH: path.resolve(cwd, 'node_modules/@jiangweiye/tsconfig/tsconfig.alias.json'),
        JSON_CONFIG_PATH: path.resolve(cwd, 'jsconfig.json'),
        TS_CONFIG_PATH: path.resolve(cwd, 'tsconfig.json')
    };
});

var getPluginConfig = function (root, defaultConfig) {
    var _a;
    return _a = {},
        _a[PLUGIN_NAME] = {
            root: path.join(root, 'src'),
            prefix: defaultConfig.prefix,
            mode: defaultConfig.mode
        },
        _a;
};
function genAlias(root, prefix) {
    var dirs = getDirs(root);
    return dirs.reduce(function (result, item) {
        var key = "".concat(prefix).concat(item.dirName);
        var value = item.dirPath;
        result[key] = value;
        return result;
    }, {});
}
function alias(api, options) {
    var _a = options.pluginOptions, pluginOptions = _a === void 0 ? {} : _a;
    var cwd = api.getCwd();
    var _b = getPaths(cwd), ALIAS_JSON_PATH = _b.ALIAS_JSON_PATH, JSON_CONFIG_PATH = _b.JSON_CONFIG_PATH, TS_CONFIG_PATH = _b.TS_CONFIG_PATH;
    var _c = mergeConfig(pluginOptions, getPluginConfig(cwd, defaultConfig(cwd)))[PLUGIN_NAME], mode = _c.mode, prefix = _c.prefix, root = _c.root;
    if (!hasFile(root)) {
        return;
    }
    api.configureWebpack(function (config) {
        var baseAlias = {};
        if (config.resolve && config.resolve.alias) {
            baseAlias = config.resolve.alias;
        }
        var _alias = __assign(__assign({}, genAlias(root, prefix)), baseAlias);
        syncJson({
            extendJson: ALIAS_JSON_PATH,
            jsJson: JSON_CONFIG_PATH,
            tsJson: TS_CONFIG_PATH,
            alias: _alias,
            root: root,
            prefix: prefix,
            mode: mode
        });
        return {
            resolve: {
                alias: _alias
            }
        };
    });
}

module.exports = alias;
